/* 
 * Copyright 2016 The Regents of the University of California
 * All rights reserved.
 * 
 * This file is part of Spoofer.
 * 
 * Spoofer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Spoofer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Spoofer.  If not, see <http://www.gnu.org/licenses/>.
 */

/****************************************************************************
   Program:     $Id: port.h,v 1.4.2.3 2016/10/13 19:08:21 kkeys Exp $
   Date:        $Date: 2016/10/13 19:08:21 $
   Description: general portability
****************************************************************************/

#ifndef COMMON_PORT_H
#define COMMON_PORT_H 1

#ifdef __cplusplus
# include <cstddef>
# if __cplusplus >= 201103L // C++11
#  define NO_METHOD = delete
# else // pre C++11
#  ifndef nullptr
    class NULLPTR_T {
	// Like nullptr in C++11, this is convertable to any type of null
	// pointer, but NOT to an integer.  So, e.g., with overloads f(int)
	// and f(char*), calling f(nullptr) will correctly choose the latter.
    public:
	NULLPTR_T() {} // C++03 std (and some compilers) requires this
	template<class T> operator T*() const { return static_cast<T*>(0); }
	template<class C, class T> operator T C::*() const { return static_cast<T*>(0); }
    };
    static const NULLPTR_T nullptr;
#  endif
#  define NO_METHOD /* declaration prevents compiler-generated default definition */
# endif
#endif

#ifdef __GNUC__
# define NORETURN                 __attribute__((__noreturn__)) void
# define ATR_UNUSED               __attribute__((__unused__))
# define ATR_USED                 __attribute__((__used__))
# define ATR_PURE                 __attribute__((__pure__))
# define ATR_CONST                __attribute__((__const__))
# define ATR_NONNULL(args)        __attribute__((__nonnull__ args))
# if !defined(__clang__) && __GNUC__ == 5 && __GNUC_MINOR__ == 1
   // silence -Wsuggest-attribute=format in gcc 5.1
#  define FORMAT_PRINTF(fmt, var) __attribute__((__format__(gnu_printf, fmt, var)))
# else
   // prefer warnings for standard printf format
#  define FORMAT_PRINTF(fmt, var) __attribute__((__format__(printf, fmt, var)))
# endif
#else
# define NORETURN                 void
# define ATR_UNUSED               /* empty */
# define ATR_USED                 /* empty */
# define ATR_PURE                 /* empty */
# define ATR_CONST                /* empty */
# define ATR_NONNULL(args)        /* empty */
# define FORMAT_PRINTF(fmt, var)  /* empty */
#endif

#endif // COMMON_PORT_H
